#include <glew/glew.h>
#include "myglwidget.h"
#include "scene.h"
#include "glassert.h"
#include "camera.h"
#include "sphere.h"
#include "phongmaterial.h"
#include "skybox.h"
#include "utils.h"
#include "texture.h"
#include "sparrowrenderer.h"
#include <iostream>
#include <glm/glm.hpp>
#include <QKeyEvent>
#include <QMouseEvent>

MyGLWidget::MyGLWidget(QWidget *parent) :
    QGLWidget(parent),
    renderer(NULL)
{}

MyGLWidget::~MyGLWidget()
{
    if(renderer != NULL)
        delete(renderer);
}

Scene* MyGLWidget::buildScene()
{
    Scene* scene = new Scene();
    Camera* cam = new Camera(width(), height());
    cam->moveTo(glm::vec3(0, 0, 3));
    cam->lookAt(glm::vec2(0, 0));
    scene->setCamera(cam);
    Mesh* myGrid = new Sphere(2);
    myGrid->initGL();
    std::string vertSource = Utils::fileToString("../phong.vert");
    std::string fragSource = Utils::fileToString("../phong.frag");
    Shader* shader = new Shader(vertSource, fragSource);
    PhongMaterial* mat = new PhongMaterial(shader);
    Texture* tex = new Texture("../data/noise.png");
    mat->setTexture(tex);
    std::string filenames[6] = {
        "../data/skybox_ft", "../data/skybox_bk",
        "../data/skybox_up", "../data/skybox_dn",
        "../data/skybox_lf", "../data/skybox_rt"
    };
    SkyBox* skybox = new SkyBox(filenames);
    scene->addEntity(skybox);
    scene->addEntity(myGrid, mat);
    return scene;
}

void MyGLWidget::initializeGL()
{
    makeCurrent();
    renderer = new SparrowRenderer(width(), height());
    Scene* scene = buildScene();
    renderer->setScene(scene);
    controller.setScene(scene);
}

void MyGLWidget::resizeGL(int width, int height)
{
    renderer->resize(width, height);
    updateGL();
}

void MyGLWidget::paintGL()
{
    renderer->render();
}

void MyGLWidget::mouseMoveEvent(QMouseEvent *e)
{
    controller.mouseMove(e->pos().x() - last.x(), e->pos().y() - last.y());
    last = e->pos();
    updateGL();
}

void MyGLWidget::mousePressEvent(QMouseEvent* e)
{
    controller.mouseEvent(e->button(), true);
    last = e->pos();
    updateGL();
}
void MyGLWidget::mouseReleaseEvent(QMouseEvent* e)
{
    controller.mouseEvent(e->button(), false);
    updateGL();
}

void MyGLWidget::keyPressEvent(QKeyEvent *e)
{
    controller.mouseEvent(e->key(), true);
    updateGL();
}
void MyGLWidget::keyReleaseEvent(QKeyEvent *e)
{
    controller.mouseEvent(e->key(), false);
    updateGL();
}
